/****************************************************************************
 * FSextend.h    header file for FlagShip Extend System                     *
 *                                                                          *
 * Release 4.40                                                             *
 *--------------------------------------------------------------------------*
 * (c) 1998 multisoft Datentechnik GmbH, Munich. All rights reserved.       *
 ****************************************************************************/

#ifndef __FGS_FLAGSHIP__
#  include "FlagShip.h"
#endif

#ifndef   __FGS_EXTEND__
#  define __FGS_EXTEND__

extern int gl_argc;
extern FSvar **gl_argv, *FALSE_VAR;

static FSvar *_FS_v_extend_generic;


#define FSinit()\
         _FS_v_extend_generic = FALSE_VAR;\
         gl_argc = argc; gl_argv = argv

/* --- FlagShip Extend System macros, macro accessable also w/o EXTERNAL -- */

#ifndef FSudfname
#ifdef _FGS_CCNAME_CONV2
#  define FSfilename(name)          _FSinit_/**/name() {}
#  define fstmpFnNAME(name)         _bb_/**/name
#  define fstmpFnLOCATOR(name)                         \
      extern FSvar * fstmpFnNAME(name)();              \
      static struct fsuFuncLocate _sfl_/**/name =      \
          { "FSfntab", "name", fstmpFnNAME(name) }
#else
#  define FSfilename(name)          _FSinit_##name() {}
#  define fstmpFnNAME(name)         _bb_##name
#  define fstmpFnLOCATOR(name)                        \
      extern FSvar * fstmpFnNAME(name)();             \
      static struct fsuFuncLocate _sfl_##name =       \
          { "FSfntab", #name, fstmpFnNAME(name) }
#endif

#define FSudfname(name)                        \
    fstmpFnLOCATOR(name);                      \
    FSvar * fstmpFnNAME(name)(argc, argv)      \
    int argc;                                  \
    FSvar * argv[];
#define FSudfCALL(name) fstmpFnName(name)
#endif


/* --- Old Extend System macros for compatibility, don't use anymore ----- */

#define Fsfilename_cpp(name)  name() {}
#define Fsudfname_cpp(x)      FSvar * x(argc, argv)\
                              int argc; FSvar *argv[];

#ifdef _FGS_CCNAME_CONV2
# define FStabdef(name)  _FSinit_/**/name() {}
# define FlagShip(x)      FSvar *_bb_/**/x(argc, argv)\
                          int argc; FSvar *argv[];
#else
# define FStabdef(name)  _FSinit_##name() {}
# define FlagShip(x)      FSvar *_bb_##x(argc, argv)\
                          int argc; FSvar *argv[];
#endif

#define Fstabdef_(name)  name() {}
#define Flagship_(x)     FSvar * x(argc, argv)\
                         int argc; FSvar *argv[];

/* --- data types --- */

#define UNDEF     0
#define CHARACTER 1
#define NUMERIC   2
#define LOGICAL   4
#define DATE      8
#define MEMO      65
#define MPTR      32       /* or'ed into the paremeter type */
#define ARRAY     512
#define SCREEN    1024
#define OBJECT    2048
#define BLOCK     4096
#define ERROR     65535

/* --- macros for input --- */

#define PCOUNT          (_parinfo(0))
#define ISCHAR(order)   (_parinfo(order) & CHARACTER)
#define ISNUM(order)    (_parinfo(order) & NUMERIC)
#define ISLOG(order)    (_parinfo(order) & LOGICAL)
#define ISDATE(order)   (_parinfo(order) & DATE)
#define ISMEMO(order)   (_parinfo(order) & MEMO)
#define ISARRAY(order)  (_parinfo(order) & ARRAY)
#define ISSCREEN(order) (_parinfo(order) & SCREEN)
#define ALENGTH(order)  (_parinfa(order, 0))
#define ISBYREF(order)  (_parinfo(order) & MPTR)
#define ISOBJECT(order) (_parinfo(order) & OBJECT)
#define ISBLOCK(order)  (_parinfo(order) & BLOCK)


/* --- declaration of extend functions for input --- */

unsigned char *_parc();
char          *_pards();
int           _parclen();
int           _parinfo(), _parinfa(), _parl(), _parni();
double        _parnd();
long          _parnl();
WINDOW *      _parscw();
int        _parcsize();
#define _parcsiz _parcsize
/* --- Memory allocation functions ---- */

/* IMPORTANT NOTE:
   do never mix malloc/free with the following  macros.
   The pointers returned differ, so their mixed usage will cause
   segmentation violations.
   The FS_x functions serve to find illegal memory overruns
   and illegal free()-calls, so they help to avoid segmentation
   violations
*/

#define _exmgrab(siz)      _FS_xalloc(siz, 1)
#define _exmback(ptr, siz) _FS_xfree(ptr)
#define _xalloc(siz)       _FS_xalloc(siz, 0)
#define _xgrab(siz)        _FS_xalloc(siz, 1)
#define _xfree(ptr)        _FS_xfree(ptr)

void * _FS_xalloc();
void   _FS_xfree();

/* --- declaration of return functions --- */

FSvar *_FSret(),   *_FSretc(),  *_FSretclen();
FSvar *_FSretds(), *_FSretl(),  *_FSretni();
FSvar *_FSretnl(), *_FSretnd(), *_FSretwin();

/* --- declaration of store  functions --- */
 int _storc(),  _storclen(), _stords(), _storl();
 int _stornd(), _storni(),   _stornl(), _storscw();

/* --- definiton of macros for return functions --- */

#define _ret()                (_FS_v_extend_generic = _FSret())
#define _retc(string)         (_FS_v_extend_generic = _FSretc(string))
#define _retclen(string, len) (_FS_v_extend_generic = _FSretclen(string, len))
#define _retds(string)        (_FS_v_extend_generic = _FSretds(string))
#define _retl(flag)           (_FS_v_extend_generic = _FSretl(flag))
#define _retnd(n)             (_FS_v_extend_generic = _FSretnd(n))
#define _retni(n)             (_FS_v_extend_generic = _FSretni(n))
#define _retnl(n)             (_FS_v_extend_generic = _FSretnl(n))
#define _retscw(n)            (_FS_v_extend_generic = _FSretwin(n))
#define FSreturn              return(_FS_v_extend_generic)
#define FSend                 return(_FS_v_extend_generic)
#ifndef FSreturn
#  define FSreturn()         return(_FS_v_extend_generic)
#  define FSend()            return(_FS_v_extend_generic)
#endif

#endif                       /* ifndef __FGS_EXTEND__  */
/* EOF  FSextend.h */
