** BENCH1.PRG ** Sieve of Erotosthenes, using numeric array or string

// Notes:
// This program runs on DOS using Clipper 5.x
// or on any UNIX using FlagShip 4.3
// For DOS programs running Fox or dBASE, use BENCH1FX.PRG
//
// Compile on DOS:  CLIPPER  bench1 -L  ; RTLINK|BLINKER bench1 ; bench1
// Compile on UNIX: FlagShip bench1.prg -nl -nL -nd -obench1 ; bench1


#ifndef FlagShip
#define LOCAL_LONG LOCAL
#define SECONDSCPU SECONDS
#else

#Cinline
#define ARRSIZE  501
#define SIZEC    500
static int cflags [ARRSIZE];
#endCinline
#endif

#define ITERATIONS 500
#define SIZE       500

LOCAL_LONG iters := ITERATIONS
LOCAL_LONG count := 0
LOCAL_LONG prime := 0
LOCAL_LONG i, j, k

LOCAL flags [SIZE], sflags
LOCAL duration, start

* CLEAR

#ifndef FlagShip
   ? 'Clipper 5.x (Locals)'
#else
   ? 'FlagShip (Locals + Typed)'
#endif
?? " Sieve of Erotosthenes,", ltrim(str(ITERATIONS)), "iterations"

? "1. Test for numeric calculations"
************************************
start = SECONDSCPU()

FOR k := 1 TO ITERATIONS
   count := 0
   FOR i := 1 TO SIZE
      flags[i] = 1
   NEXT
   FOR i := 1 TO SIZE
      IF flags[i] != 0
         prime = 2 * i + 3
         j     = i + prime
         WHILE j <= SIZE
            flags[j] = 0
            j += prime
         ENDDO
         count++
      ENDIF
   NEXT
NEXT

duration = SECONDSCPU() - start
? '   Number of primes were '
?? LTRIM(STR(count)) + ".  "
?? LTRIM(STR(iters)), 'iterations took '
   dispout(transform(duration, '999.99'), "W+")
?? ' seconds'

? "2. Test for string handling"
*********************************
sflags := replicate(CHR(1), SIZE)

start = SECONDSCPU()

FOR k := 1 TO ITERATIONS
   count := 0
   FOR i := 1 TO SIZE
      IF STRPEEK (@sflags,i) != 2
         prime = 2 * i + 3
         j     = i + prime
         WHILE j <= SIZE
            STRPOKE (@sflags, j, 2)
            j += prime
         ENDDO
         count++
      ENDIF
   NEXT
NEXT

duration = SECONDSCPU() - start
? '   Number of primes were '
?? LTRIM(STR(count)) + ".  "
?? LTRIM(STR(iters)), 'iterations took '
   dispout(transform(duration, '999.99'), "W+")
?? ' seconds'

#ifdef FlagShip
? "3. Test for C-inline usage"
*****************************
start = SECONDSCPU()

FOR k := 1 TO ITERATIONS
   count := 0
#Cinline
   for (i = 0; i <= SIZEC; i++)
      cflags[i] = 1;

   for (i = 0; i <= SIZEC; i++) {
      if (cflags[i]) {
         prime = 2 * i + 3;
         j     = i + prime;
         while (j <= SIZEC) {
            cflags[j] = 0;
            j += prime;
         }
         count++;
      }
   }
#endCinline
NEXT

duration = SECONDSCPU() - start
? '   Number of primes were '
?? LTRIM(STR(count)) + ".  "
?? LTRIM(STR(iters)), 'iterations took '
   dispout(transform(duration, '999.99'), "W+")
?? ' seconds'
#endif

* wait

?
? "Guideline values for identical HW :"
?
set color to "W+"
? "a) 80486DX50, 16MB, DOS5 vs. SCO UNIX :"
? "                                         Clipper 5.x  FlagShip 4.3   "
DISPOUT ("faster", "BG+")
set color to "W"
? "1. numer.calculation,   200 interations     4.78 sec    0.92 sec  = "
DISPOUT ("  5.2 x", "BG+")
? "2. string manipulation, 200 interations    41.19 sec    1.91 sec  = "
DISPOUT (" 21.5 x", "BG+")
? "3. num.calc, C-inline,  200 interations     4.78 sec    0.04 sec  = "
DISPOUT ("119.5 x", "BG+")
?
set color to "W+"
? "b) PP200, 64MB, Win-NT4 vs. Linux or SCO :"
? "                                         Clipper 5.3  FlagShip 4.4   "
DISPOUT ("faster", "GR+")
set color to "W"
? "1. numer.calculation,   500 interations     9.61 sec    1.27 sec  = "
DISPOUT ("  7.6 x", "GR+")
? "2. string manipulation, 500 interations    76.35 sec    2.51 sec  = "
DISPOUT (" 30.4 x", "GR+")
? "3. num.calc, C-inline,  500 interations     9.61 sec    0.07 sec  = "
DISPOUT ("137.2 x", "GR+")
?

* set color to "W+"
* ? "c) PP200, 64MB, Win-NT4 vs. Linux or SCO :"
* ? "                                          FoxPro 2.6  FlagShip 4.4   "
* DISPOUT ("faster", "GR+")
* set color to "W"
* ? "1. numer.calculation,   500 interations     4.58 sec    1.27 sec  = "
* DISPOUT ("  3.6 x", "GR+")
* ? "2. string manipulation, 500 interations    45.36 sec    2.51 sec  = "
* DISPOUT (" 18.1 x", "GR+")
* ? "3. num.calc, C-inline,  500 interations     4.58 sec    0.07 sec  = "
* DISPOUT (" 65.4 x", "GR+")
* ?

#ifdef FlagShip
if usersmax() == 4
  ?
  dispout("Note:", "r+") ; qqout(" the used FlagShip DEMO version is ")
  dispout("slower", "R+"); qqout(" than the licensed version,")
  ? "      so don't use it for true benchmarks "
  dispout(":-)", "R+")
endif
#endif

wait

#ifndef FlagShip
   FUNCTION strpeek (flags, pos)
   RETURN ASC( SUBSTR(flags, pos, 1))

   FUNCTION strpoke (flags, pos, val)
   flags := STUFF (flags, pos, 1, CHR(val))
   RETURN flags
#endif

** eof **

