*** File: creadb.prg
***
*** Compile: FlagShip creadb.prg -ocreadb

*----------------------------------------------------------------------
* Main program
*
* FUNCTION Start (DbfName, AsciName)    // retr. command-line params
PARAMETERS DbfName, AsciName            // alternative syntax
LOCAL executable
LOCAL aDbStru := {}

#ifdef FlagShip                         // FlagShip settings
# include "fspreset.fh"                 // convert files to lowercase
# include "fileio.fh"                   // defines used here
  executable := execname()
#else                                   // Clipper 5.x settings
# include "fileio.ch"
  executable := "EXAMPLE5.EXE"
# define FS_START 0
#endif

if .not. CheckIt (@DbfName, AsciName, executable)
   quit
endif
aDbStru := readStruct(AsciName)          // fill array
if len(aDbStru) == 0
   ? "** sorry, no valid data in the file " + AsciName
   ? "   invoke '" + executable + " -h' for help"
   quit
endif

dbcreate (DbfName, aDbStru)              // create the database file

?
? "--> done, file " + lower(DbfName)
if ascan(aDbStru, {|x| x[2]=="M" }) > 0  // memo field(s) ?
   ?? " and " + lower(substr(DbfName,1,len(DbfName)-1)) + "t"
endif
?? " created."
? "--> Display structure (y/j/o|n) ? " 
ii := 0
while !(upper(chr(ii)) $ "YJNON")
   ii := inkey(0)
enddo
if upper(chr(ii)) # "N"
   displStruct (DbfName)
endif
return 0

*----------------------------------------------------------------------
* CheckIt() checks and corrects the parameters passed by reference
*
FUNCTION CheckIt (DbfName, AsciName, execName)
LOCAL answer := ""
? "--> This '" + execName + "' creates a database according to " + ;
  "ASCII structure file"

if empty(DbfName) .or. empty(AsciName) .or. left(dbfName,2) == "-h"
   ? "--> The correct syntax is: " + execName + ;
     " <dbfName> <ascName>"
   ? "    where <dbfName> is the name of the dbf (optional with path)"
   ? "          <ascName> is the name of an ASCII file " + ;
     "describing the dbf structure"
   ? "    Structure of each line in the <ascName> file:"
   ? "    <FieldName> <FieldType> [<FieldLen>] [<FieldDeci>]"
   ? "      <FieldName> is the name of the field"
   ? "      <FieldType> is the field type (C,N,D,L,M)"
   ? "      <FieldLen>  is the total field length [or default]" 
   ? "      <FieldDeci> is the number of decimal digits [or 0]"
   ? "    whereby the data are separated by space(s) or tab(s), e.g.:"
   ? "      |# comment lines or inline-comments are prefaced by the '#' sign"
   ? "      |                     # empty lines are ignored"
   ? "      |FamName C            # sets default field lenght to 10"
   ? "      |AddRess ch  50       # uppler/lower case is supported"
   ? "      |Salary  Num 10  3    # only 1st char of FieldType is signif."
   ? "      |# eof"
   ?
   return .F.
else
   ? "--> enter '" + execName + " -h' for help"
   ? "-->"
   ? "--> Your entry is: " + execName, DbfName, AsciName
   ?
endif
if .not. file(AsciName)
   ? "*** Sorry, the ascii file " + asciName + " is not available."
   return .F.
endif
if .not. (upper(right(dbfName,4)) == ".DBF")
   dbfName += ".dbf"                        // make currection
endif
if file(dbfName)
   while !(upper(left(answer,1)) $ "NYJO")  // wait for correct key
      accept "Warning: database " + dbfName + " is AVAILABLE. " + ;
             "Overwrite (N|no/y|yes|j|o) ? " to answer
   enddo
   if upper(left(answer,1)) == "N"
      return .F.
   endif
endif
return .T.

*----------------------------------------------------------------------
* ReadStruct() reads the ascii file and stores data into array
*
STATIC FUNCTION ReadStruct (cFileName)
LOCAL  ii, handle, buff, nSize, line := 0, retArr := {}, arr

handle := FOPEN (cFileName, FO_READ)     // open input read-only
if handle < 0
   ? "*** i/o error on open " + cFileName + " (access rights ?)"
   return retArr
endif
nSize := FSEEK (handle, 0, FS_END)       // file size in bytes
FSEEK (handle, 0, FS_START)              // go to file begin

while FSEEK(handle, 0, FS_RELATIVE) < nSize
   buff := alltrim( FREADTXT (handle))   // read one asci line
   buff := strtran (buff, chr(9), " ")   // convert tabs to space
   ? str(++line,3) + ":" + buff
   if (ii := at("#", buff)) > 0
      buff := substr(buff, 1, ii-1)      // remove comments
   endif
   if empty(buff)                        // empty lines are ignored
      loop                               // read next line
   endif
   arr := tokenize (@buff, retArr)
   aadd (retArr, arr)
enddo
FCLOSE (handle)                          // close input
return retArr

*----------------------------------------------------------------------
* Tokenize() split input into tokens
*
STATIC FUNCTION Tokenize (buff, retArr)
LOCAL  ii, cc, nType := 0, arr := {}
STATIC aCheck := {{"C",0}, {"N",0}, {"M",10}, {"D",8}, {"L",1}}

while "  " $ buff
   buff := strtran (buff, "  ", " ")     // remove multiple spaces
enddo
buff := upper(buff) + " "                // convert to uppercase
while (ii := at(" ", buff)) > 1          // tokenize the input
   aadd (arr, left(buff, ii-1))          // into array elements
   buff := substr(buff, ii+1)
   if len(arr) == 4
	  exit
   endif
enddo

do case
case len(arr) < 2                        // minimal requirements ?
   ? "*** error: at least <FieldName> and <FieldType> must be given"
   quit
case !isalpha(arr[1])                    // 1st char = Alpha
   ? "*** error: <FieldName> must start with Alpha char, here: " + arr[1]
   quit
endcase
arr[1] := trim(left(arr[1], 10))         // name = max 10 chars

for ii := 1 to len(arr[1])               // check name validity
   cc := substr(arr[1],ii,1)
   if .not. (isAlpha(cc) .or. isDigit(cc) .or. cc == "_")
      ? "*** error: invalid character '" + cc + "' in " + arr[1]
      quit
   endif
next
if len(retArr) > 0
   if ascan (retArr, {|x| x[1] == arr[1]}) > 0  // check field name
      ? "*** error: field name " + arr[1] + " is multiple defined"
      quit
   endif
endif

for ii := 1 to len(aCheck)
   if aCheck[ii,1] == left(arr[2],1)     // check the field type
      nType := ii
      exit
   endif
next
if nType == 0
   ? "*** error: wrong <FieldType>, here: " + arr[2]
   quit
endif
arr[2] := left(arr[2],1)

if len(arr) < 3                          // check the field size
   aadd (arr, if(aCheck[nType,2] == 0, 10, aCheck[nType,2]))
   ? "--- FieldLength of " + arr[1] + " set to " + ltrim(str(arr[3]))
else
   arr[3] := val(arr[3])
   if arr[3] <= 0 .or.  ;
      (arr[3] # aCheck[nType,2] .and. aCheck[nType,2] # 0)
      ? "--- FieldLength of " + arr[1] + " CORRECTED " + ;
        "from " + ltrim(str(arr[3]))
      arr[3] := if(aCheck[nType,2]==0, 10, aCheck[nType,2])
      ?? " to " + ltrim(str(arr[3])) + " ... any key"
      inkey(0)
   endif
endif
if len(arr) < 4                          // check deci places
   aadd (arr, 0)
else
   arr[4] := val(arr[4])
   if arr[2]=="N"                        // but for numeric only
      if arr[4] < 0
         arr[4] := 0
      else
         if (arr[4] +2) > arr[3]
            ? "--- DeciPlaces of " + arr[1] + " CORRECTED " + ;
              "from " + ltrim(str(arr[4]))
            arr[4] := arr[3] -2
            ?? " to " + ltrim(str(arr[4])) + " ... any key"
            inkey(0)
         endif
      endif
   else
      arr[4] := 0
   endif
endif
return arr

*----------------------------------------------------------------------
* DisplStruct() display the database structure
*
static function displstruct(name)
local astru, ii, jj, m1 := 0, m2 := 0, m3 := 0, split := .F., dummy
use (name)
astru := dbstruct()
use
dummy := "    ------------- structure of " + name + " "
dummy += replicate("-", 60 - len(dummy) -1)
? dummy
ii := len(astru)
if ii > 5
   ii := round((ii / 2) + 0.1, 0)
   split := .T.
endif
for jj := 1 to len(astru)
   m1 := max(m1, len(astru[jj,1]))
   m2 := max(m2, len(ltrim(str(astru[jj,3]))))
   m3 := max(m3, len(ltrim(str(astru[jj,4]))))
next
for jj := 1 to ii
   ? space(4) + padr(astru[jj,1], m1), astru[jj,2], ;
	 str(astru[jj,3],m2), str(astru[jj,4],m3)
   if split .and. ((jj + ii) <= len(astru))
      ?? space(10), padr(astru[jj+ii,1], m1), astru[jj+ii,2], ;
		 str(astru[jj+ii,3],m2), str(astru[jj+ii,4],m3)
   endif
next
? "    " + replicate("-", 60 -5)
return NIL

*----------------------------------------------------------------------
* For DOS: simulate FlagShip functions not available in Clipper
*
#ifndef FlagShip
  static FUNCTION FREADTXT(handle)
  local buff := space(500), out := ""
  local pos, ii, jj, cc
  pos := FSEEK (handle, 0, FS_RELATIVE)
  jj := FREAD (handle, @buff, 500)
  for ii := 1 to jj
     cc := substr(buff,ii,1)
     if asc(cc) != 10 .and. asc(cc) != 13
        out += cc
        pos++
     else
        while ii <= jj .and. (cc == chr(10) .or. cc == chr(13))
           pos++
           cc := substr(buff,++ii,1)
        enddo
        exit
     endif
  next
  FSEEK (handle, pos, FS_START)
  return out
#endif

*** eof
