/****************************************************************************
 * GETSYS.PRG standard Clipper 5.x and FlagShip 4.4x GET/READ system        *
 *                                                                          *
 * FlagShip     : compile local  FlagShip getsys.prg -m -na -c              *
 *                link with appl FlagShip myprog*.prg getsys.o ...          *
 * Clipper 5.x  : compile with   CLIPPER  getsys.prg /m/n/w                 *
 *--------------------------------------------------------------------------*
 * Copyright (C) 1990-1992 Computer Associates International, Inc.          *
 * Commented, modified and adopted for FlagShip (c) 1998 by multisoft GmbH. *
 *    essenc.differences in  #ifdef FlagShip  blocks only                   *
 ****************************************************************************/

#define C87_COMPATIBEL             /* set compatibility to Clipper'87       */
/* #define GERMAN_MSG              /* activate for german scoreboard msg    */
/* #define CTRL_END_SPECIAL        /* activate for ^W and ^End = last GET   */
/* #define SUBSCRIPT_IN_READVAR    /* deactivate for READVAR() like C87     */

#ifdef C87_COMPATIBEL
#  define EMPTY_DATE_VALID         /* C87: empty date is valid, C5: invalid */
#endif

#ifdef FlagShip
#  include "set.fh"
#  include "inkey.fh"
#  include "getexit.fh"
#  include "getclass.fh"
#  define K_UNDO             K_CTRL_U
#else
#  include "Set.ch"
#  include "Inkey.ch"
#  include "Getexit.ch"
#  define K_UNDO             K_ALT_U
#endif

** #define ADD_FS430
// Scoreboard messages
// FlagShip: user defined messages taken from FSsortab.xxx,
//           if FS_SET("loadlang") and FS_SET("setlang") called.
//           See also RangeCheck(), ShowScore(), DateMsg()

#ifdef GERMAN_MSG
#  define GSMSG_INSERT  "Einfg."
#  define GSMSG_RANGE   "Bereich: "
#  define GSMSG_INVDATE "unglt. Datum"
#else
#  define GSMSG_INSERT  "Insert"
#  define GSMSG_RANGE   "Range is "
#  define GSMSG_INVDATE "Invalid date "
#endif

// display coordinates for SCOREBOARD

#define SCORE_ROW    0
#define SCORE_COL    60

// STATIC variables for the active READ

static Format
static Updated  := .f.
static KillRead := .f.
static BumpTop  := .f.
static BumpBot  := .f.
static LastExit := 0
static LastPos  := 0
static ActiveGet
static ReadProcName := ""
static ReadProcLine := ""

// Array elements to store actual settings

#define GSV_KILLREAD       1
#define GSV_BUMPTOP        2
#define GSV_BUMPBOT        3
#define GSV_LASTEXIT       4
#define GSV_LASTPOS        5
#define GSV_ACTIVEGET      6
#define GSV_READVAR        7
#define GSV_READPROCNAME   8
#define GSV_READPROCLINE   9
#define GSV_COUNT          9

proto func GetPreValidate(get  AS GET) AS LOGICAL
proto func GetPostValidate(get AS GET) AS LOGICAL

/*--------------------------------------------------------------------------*
 | READMODAL()  is the "old" READ                                           |
 |                                                                          |
 | in: p1 (Array) with GET objects                                          |
 | fn: updated status (.T./.F.)                                             |
 *--------------------------------------------------------------------------*/
func ReadModal ( GetList AS ARRAY ) AS USUAL

local get as get
local pos
local savedGetSysVars, cOut

   if ( ValType(Format) == "B" )                      // SET FORMAT TO ?
      Eval(Format)                                    // yes, process
   end

   if ( Empty(getList) )                              // S87 compat.
      SetPos( MaxRow()-1, 0 )                         // no READ without obj
      return (.f.)
   end

#ifdef FlagShip
   cOut := _LANGNAMES(1)
#else
   cOut := GSMSG_INSERT
#endif

   savedGetSysVars := ClearGetSysVars()               // save act.status

   ReadProcName := ProcName(1)                        // save calling UDF
   ReadProcLine := ProcLine(1)                        // for SET KEY

   pos := Settle( Getlist, 0 )                        // first GET

   while ( pos <> 0 )
      get := GetList[pos]                             // catch one GET obj
      PostActiveGet( get )                            // activate it

      if ( ValType( get:reader ) == "B" )             // user READER block ?
         Eval( get:reader, get )                      // - yes, execute
      else
         GetReader( get )                             // - no, build in
      end

      if len(GetList) < pos
         KillRead := .T.                              // GetList was killed
         exit                                         // in external UDF
      end
      pos := Settle( GetList, pos )                   // next GET
   end

#ifdef C87_COMPATIBEL
   if ( Set(_SET_SCOREBOARD) .and. Set(_SET_INSERT))  // SET SCOREBOARD ON
      SetPos(SCORE_ROW, SCORE_COL)                    // and INSERT active ?
      DispOut( SPACE( LEN( cOut ) ))                  // yes, clear scoreboard
   end
#endif

   RestoreGetSysVars(savedGetSysVars)                 // restore status
   SetPos( MaxRow()-1, 0 )                            // S87 compat.

return (Updated)

/*--------------------------------------------------------------------------*
 | GETREADER()  standard READER to process ONE Get (object)                 |
 |              called from READMODAL()                                     |
 |                                                                          |
 | in: GET object                                                           |
 | fn: -                                                                    |
 *--------------------------------------------------------------------------*/
proc GetReader( get AS GET ) AS USUAL

#ifdef ADD_FS430
   local stat := fs_set ("zero", .F.)
   if !empty(get:name)
      get:name := Upper(substr(get:name, 1, LEN(get:name)))
   endif
   fs_set ("zero", stat)
#endif

   if ( GetPreValidate(get) )                         // WHEN option ok?
      get:SetFocus()                                  // activate object

      while ( get:exitState == GE_NOEXIT )            // loop until exit
         if ( get:typeOut )                           // editable position ?
            get:exitState := GE_ENTER                 // - no, simul.RETURN
         end

         while ( get:exitState == GE_NOEXIT )
            GetApplyKey( get, Inkey(0) )              // READ next key
         end

         if ( !GetPostValidate(get) )                 // check VALID block
            get:exitState := GE_NOEXIT                // invalid: stay here
         end
      end

      get:KillFocus()                                 // deactivate object
   end
return

/*--------------------------------------------------------------------------*
 | GETAPPLKEY() check input key, take action on object                      |
 |              note: the actual object should have Focus set               |
 |              called from GETREADER()                                     |
 |                                                                          |
 | in: GET object                                                           |
 |     key (numer.equivalent) from INKEY                                    |
 *--------------------------------------------------------------------------*/
proc GetApplyKey (get AS GET, key AS INTVAR)

local cKey
local bKeyBlock

   if ( (bKeyBlock := SetKey(key)) <> NIL )           // SET KEY active ?
      GetDoSetKey(bKeyBlock, get)                     // yes, process UDF
      return                                          // and go back for
   end                                                // next key

#ifdef FlagShip
   if ( len(ckey := __Getfunction(key)) <> 0 )        // SET Function active ?
      keyboard ckey                                   // yes, Keyboard it
      return                                          // and go back for
   end                                                // next key
#endif

   do case
   case ( key == K_UP )                               // cursor up:
      get:exitState := GE_UP                          //    previous GET

   case ( key == K_SH_TAB )                           // shift TAB:
      get:exitState := GE_UP                          //    previous GET

   case ( key == K_DOWN )                             // cursor down:
      get:exitState := GE_DOWN                        //    next GET

   case ( key == K_TAB )                              // TAB:
      get:exitState := GE_DOWN                        //    next GET

   case ( key == K_ENTER )                            // RETURN:
      get:exitState := GE_ENTER                       //    next GET

   case ( key == K_ESC )                              // ESCAPE:
      if ( Set(_SET_ESCAPE) )                         //    SET ESCAPE ON ?
         get:undo()                                   //    yes, undo
         get:exitState := GE_ESCAPE                   //    exit READ
      end

   case ( key == K_PGUP )                             // PageUp:
      get:exitState := GE_WRITE                       //    update + exit

   case ( key == K_PGDN )                             // PageDown:
      get:exitState := GE_WRITE                       //    update + exit

   case ( key == K_CTRL_HOME )                        // Ctrl-HOME :
      get:exitState := GE_TOP                         //    first GET

#ifdef CTRL_END_SPECIAL
   case (key == K_CTRL_END)                           // ^W and ^END :
      get:exitState := GE_BOTTOM                      //    go last GET
#else
   case (key == K_CTRL_W)                             // ^W and ^END :
      get:exitState := GE_WRITE                       //    update + exit
#endif

   case (key == K_INS)                                // INSERT:
      Set( _SET_INSERT, !Set(_SET_INSERT) )           //    set insert mode
      ShowScoreboard()                                //    show score

   case (key == K_UNDO)                               // ^U or Alt-U:
      get:Undo()                                      //    Undo

   case (key == K_HOME)                               // HOME:
      get:Home()                                      //    go first pos

   case (key == K_END)                                // END:
      get:End()                                       //    go last pos/buff

   case (key == K_RIGHT)                              // Cursor RIGHT
      get:Right()                                     //    move cursor/exit

   case (key == K_LEFT)                               // Cursor LEFT
      get:Left()                                      //    move cursor/exit

   case (key == K_CTRL_RIGHT)                         // Ctrl-Cursor RIGHT
      get:WordRight()                                 //    go next word

   case (key == K_CTRL_LEFT)                          // Ctrl-Cursor LEFT
      get:WordLeft()                                  //    go prev. word

   case (key == K_BS)                                 // BACKSPACE
      get:BackSpace()                                 //    delete prev.char

   case (key == K_DEL)                                // DELETE
      get:Delete()                                    //    delete act.char

   case (key == K_CTRL_T)                             // Ctrl-T
      get:DelWordRight()                              //    delete word right

   case (key == K_CTRL_Y)                             // Ctrl-Y
      get:DelEnd()                                    //    delete to buff end

   case (key == K_CTRL_BS)                            // Ctrl-BACKSPACE
      get:DelWordLeft()                               //    delete word left

   otherwise
      if ((key >= 32 .and. key <= 255) .or. ;         // char 32..255
          (key == K_CTRL_U .and. key != K_UNDO))      // or paragraph sign
         cKey := Chr(key)                             //    save char

         if (get:type $ "NFI" .and. (cKey == "." .or. cKey == ","))
            get:ToDecPos()
         else
            if ( Set(_SET_INSERT) )                   // INSERT on ?
               get:Insert(cKey)                       //    yes, insert char
            else
               get:Overstrike(cKey)                   //    no, type over
            end
            if ( get:typeOut )                        // out of buffer ?
               if ( Set(_SET_BELL) )                  //    SET BELL ON ?
                  ?? Chr(7)                           //    yes, bell
               end
               if ( !Set(_SET_CONFIRM) )              //    SET CONFIRM ON ?
                  get:exitState := GE_ENTER           //    no, exit GET
               end
            end
         end
      end
   endcase
return

/*--------------------------------------------------------------------------*
 | GETPREVALI() check the WHEN condition on begin of GET processing         |
 |              called from GETREADER()                                     |
 |                                                                          |
 | in: GET object                                                           |
 | fn: .T. WHEN is ok                                                       |
 *--------------------------------------------------------------------------*/
func GetPreValidate(get AS GET) AS LOGICAL

local saveUpdated
local when := .t. AS LOGIC

   if ( get:preBlock <> NIL )                         // WHEN block available?
      saveUpdated := Updated                          // yes, save status
      when := Eval(get:preBlock, get)                 //   eval WHEN block
      get:Display()                                   //   display act. GET
      ShowScoreBoard()                                //   display score
      Updated := saveUpdated                          //   restore status
   end

   if ( KillRead )                                    // READ canceled ?
      when := .f.                                     //    no GET edit
      get:exitState := GE_ESCAPE                      //    exit READMODAL
   elseif ( !when )                                   // WHEN block not ok
      get:exitState := GE_WHEN                        //    no GET edit
   else
      get:exitState := GE_NOEXIT                      // edit the actual GET
   end
return (when)

/*--------------------------------------------------------------------------*
 | GETPOSTVAL() check the VALID condition before exiting GET                |
 |              check date validation, no GET exit on invalid date          |
 |              called from GETREADER()                                     |
 |                                                                          |
 | in: GET object                                                           |
 | fn: .T. if VALID is ok, ESCAPE was pressed or READ was killed            |
 *--------------------------------------------------------------------------*/
func GetPostValidate(get AS GET) AS LOGICAL

local saveUpdated
local changed, dateok, valid := .T.
local result := .T. AS LOGIC

   if ( get:exitState == GE_ESCAPE )                  // ESCAPE pressed ?
      return (.t.)                                    // yes, no VALID cond
   end

   dateok := !get:BadDate()                           // date value ok ?
#ifdef FlagShip
#  ifdef EMPTY_DATE_VALID
   if !dateok .and. get:type == "D"                   // FlagShip:
      dateok := get:EmptyDate                         // allow empty date
   end                                                // for C87 compatib.
#  endif
#endif
   if !dateok                                         // Date valid ?
      get:Home()                                      // no, VALID is false
      DateMsg()                                       //    prepare and
      ShowScoreboard()                                //    show scoreboard,
      return (.f.)                                    //    stay in GET
   end

   if ( get:changed )                                 // changes within GET ?
      get:Assign()                                    // yes, write buffer
      Updated := .t.                                  // into var, set update
   end
   get:Reset()                                        // redisplay buffer

   if ( get:postBlock <> NIL )                        // VALID block avail ?
      saveUpdated := Updated                          // save status
      SetPos( get:row, get:col + Len(get:buffer) )    // S87 compat. cursor

      valid := Eval(get:postBlock, get)               // eval VALID block
      while VALTYPE(valid) == "B"
         valid := Eval(valid, get)                    // eval VALID block
      end
      result := valid

      SetPos( get:row, get:col )                      // reset compat. pos
      ShowScoreBoard()                                // scoreboard
      get:UpdateBuffer()                              // restore buffer
      Updated := saveUpdated                          // restore status

      if ( KillRead .or. !get:HasFocus)               // READ killed in UDF ?
         get:exitState := GE_ESCAPE                   // exit ReadModal()
         result := .t.
         KillRead = .T.
      end
   end
return (result)

/*--------------------------------------------------------------------------*
 | GETDOSETKE() call proc according to SET KEY                              |
 |              called from GETAPPLYKEY()                                   |
 |                                                                          |
 | in: SetKey block from SetKey(key)                                        |
 |     GET object                                                           |
 *--------------------------------------------------------------------------*/
proc GetDoSetKey(keyBlock AS CODEBLOCK, get AS GET)

local saveUpdated

   if ( get:changed )                                 // if GET changed:
      get:Assign()                                    // write buffer into
      Updated := .t.                                  // variable
   end
   saveUpdated := Updated                             // save status

   Eval(keyBlock, ReadProcName, ReadProcLine, ReadVar())  // call UDF

   ShowScoreboard()                                   // show scoreboard
   get:UpdateBuffer()                                 // restore buffer
   Updated := saveUpdated                             // restore status

   if ( KillRead .or. !get:HasFocus)                  // READ killed in UDF ?
      get:exitState := GE_ESCAPE                      // exit ReadModal()
      KillRead = .T.
   end
return


/*--------------------------------------------------------------------------*
 | GETACTIVE () returns the active GET object from READ                     |
 |              standard Clipper/FlagShip function                          |
 *--------------------------------------------------------------------------*/
func GetActive(g AS GET) AS GET
local oldActive as get
   if ( ValType(ActiveGet) == 'O' )
	  oldActive := ActiveGet
   endif
   if ( PCount() > 0 .and. ValType(g) == 'O' )
      ActiveGet := g
   end
return ( oldActive )

/*--------------------------------------------------------------------------*
 | UPDATED ()   returns the global get:changed status from all prev.GETS    |
 |              standard Clipper/FlagShip function                          |
 *--------------------------------------------------------------------------*/
func Updated() AS LOGICAL
return (Updated)

/*--------------------------------------------------------------------------*
 | READEXIT()   set/reset GET exit moving cursor outside of buffer          |
 |              standard Clipper/FlagShip function                          |
 *--------------------------------------------------------------------------*/
func ReadExit ([lNew]) AS LOGICAL
return ( Set(_SET_EXIT, lNew) )

/*--------------------------------------------------------------------------*
 | READINSERT() set/reset the actual INSERT status                          |
 |              standard Clipper/FlagShip function                          |
 *--------------------------------------------------------------------------*/
func ReadInsert ([lNew]) AS LOGICAL
return ( Set(_SET_INSERT, lNew) )

/*--------------------------------------------------------------------------*
 | READKEY()    transform LastKey() codes from FlagShip/Clipper             |
 |              INKEY() value into dBASE III compatible value               |
 *--------------------------------------------------------------------------*/
#ifdef FlagShip
funct ReadKey () AS INTVAR

local lastkey := lastkey()
local add     := if (Updated, 256, 0)

do case
   case lastkey == K_RETURN                           // Enter
      return 15 + add
   case lastkey == K_ESC                              // Esc
      return 12 + add
   case lastkey == K_PGUP                             // PgUp
      return  6 + add
   case lastkey == K_PGDN                             // PgDn
      return  7 + add
   case lastkey == K_CTRL_PGUP                        // ^PgUp
      return 34 + add
   case lastkey = K_CTRL_PGDN                         // ^PgDn
      return 35 + add
   case lastkey == K_CTRL_END .or. ;                  // Ctrl-End
        lastkey == K_CTRL_W                           // Ctrl-W
      return 14 + add
   case lastkey >= 32 .or. ;
        (lastkey == K_CTRL_U .and. lastkey != K_UNDO) // type past end
      return 15 + add
   otherwise
      return 0
endcase
#endif

/*--------------------------------------------------------------------------*
 | __SETFORMA() set/reset SET FORMAT TO                                     |
 |              used in std.ch and std.fh (SET FORMAT TO)                   |
 *--------------------------------------------------------------------------*/
func __SetFormat(b)
   Format := if ( ValType(b) == "B", b, NIL )
return (NIL)

/*--------------------------------------------------------------------------*
 | __KILLREAD() kill active READ                                            |
 |              used in std.ch and std.fh (CLEAR GETS)                      |
 *--------------------------------------------------------------------------*/
proc __KillRead()
   KillRead := .t.
return

/*--------------------------------------------------------------------------*
 | RANGECHECK() check RANGE from..to                                        |
 |              used in std.ch and std.fh (RANGE clause)                    |
 |                                                                          |
 | notes: -parameter 2 is unused, for compatibility to Clipper 5.00 only    |
 |        -value will be displayed in the same format, as the PICTURE       |
 |         within the GET (incompatible to Clipper'87)                      |
 *--------------------------------------------------------------------------*/
func RangeCheck(get AS GET, junk, lo AS NUMER, hi AS NUMER) AS LOGIC

local cMsg, nRow, nCol
local xValue

   if ( !get:changed )                                // don't ckeck,
      return (.t.)                                    // if buffer unchanged
   end

   xValue := get:VarGet()

   if ( xValue >= lo .and. xValue <= hi )             // range is
      return (.t.)                                    // o.k.
   end

   if ( Set(_SET_SCOREBOARD) )                        // SET SCOREBOARD ON ?

#ifdef FlagShip
      cMsg := _LANGNAMES (2) + " "                // get msg from FSsortab.xxx
#else
      cMsg := GSMSG_RANGE
#endif

      cMsg += Ltrim(Transform(lo, get:picture)) + ;
              " - " + Ltrim(Transform(hi, get:picture))    // see note

      if ( Len(cMsg) > MaxCol() )
         cMsg := Substr( cMsg, 1, MaxCol() )
      end

      nRow := Row()                                   // display error msg
      nCol := Col()

      SetPos( SCORE_ROW, Min(60, MaxCol() - Len(cMsg)) )
      DispOut(cMsg)
      SetPos(get:Row, get:Col)

      while ( NextKey() == 0 )                        // wait for next
      end                                             // key pressed

      SetPos( SCORE_ROW, Min(60, MaxCol() - Len(cMsg)) )
      DispOut( Space(Len(cMsg)) )
      SetPos(nRow, nCol)                              // clear err msg
   end
return (.f.)


/************************** service routines only **************************/

*
* Settle()
*
* returns new element from GETLIST according to:
*
*  - the actual position
*  - the EXIT status of the actual GET
*
* Note: return 0 signals an exit of the READ
*       the EXIT status from the previous GET will be transfered forward
*
static func Settle(GetList, pos)

local exitState

   if ( pos == 0 )
      exitState := GE_DOWN
   else
      exitState := GetList[pos]:exitState
   end

   if ( exitState == NIL )                            // new, illegal GET
      return 0
   end
   if ( exitState == GE_ESCAPE .or. exitState == GE_WRITE )
      return ( 0 )                                    // NOTE
   end

   if ( exitState <> GE_WHEN )                        // reset status info
      LastPos := pos
      BumpTop := .f.
      BumpBot := .f.
   else                                               // move the last exit
      exitState := LastExit                           // exit satus
   end

   do case                                            // move GET element
   case ( exitState == GE_UP )
      pos --

   case ( exitState == GE_DOWN )
      pos ++

   case ( exitState == GE_TOP )
      pos := 1
      BumpTop := .T.
      exitState := GE_DOWN

   case ( exitState == GE_BOTTOM )
      pos := Len(GetList)
      BumpBot := .T.
      exitState := GE_UP

   case ( exitState == GE_ENTER )
      pos ++

   endcase


   if ( pos == 0 )                                    // bounce top/end
      if ( !ReadExit() .and. !BumpBot )
         BumpTop := .T.
         pos := LastPos
         exitState := GE_DOWN
      end
   elseif ( pos == Len(GetList) + 1 )
      if ( !ReadExit() .and. exitState <> GE_ENTER .and. !BumpTop )
         BumpBot := .T.
         pos := LastPos
         exitState := GE_UP
      else
         pos := 0
      end
   end


   LastExit := exitState                              // hold last exit state

   if ( pos <> 0 )
      GetList[pos]:exitState := exitState
   end

return (pos)


****************************************************
* PostActiveGet()
* Post active GET for ReadVar(), GetActive().
*
static proc PostActiveGet(get as get)

   GetActive( get )
   ReadVar( GetReadVar(get) )

   ShowScoreBoard()

return

****************************************************
* ClearGetSysVars()
* save/clear the actual status info from READ into an internal array
* note: UPDATED status will be cleared, but not restored (S87 compat.)
*
static func ClearGetSysVars()

local saved[ GSV_COUNT ]

   saved[ GSV_KILLREAD ] := KillRead
   KillRead := .f.

   saved[ GSV_BUMPTOP ] := BumpTop
   BumpTop := .f.

   saved[ GSV_BUMPBOT ] := BumpBot
   BumpBot := .f.

   saved[ GSV_LASTEXIT ] := LastExit
   LastExit := 0

   saved[ GSV_LASTPOS ] := LastPos
   LastPos := 0

   saved[ GSV_ACTIVEGET ]    := GetActive()
   saved[ GSV_READVAR ]      := ReadVar( "" )
   saved[ GSV_READPROCNAME ] := ReadProcName
   ReadProcName := ""

   saved[ GSV_READPROCLINE ] := ReadProcLine
   ReadProcLine := 0

   Updated := .f.

return (saved)

****************************************************
* RestoreGetSysVars()
* restore the actual status info READ from internal array
* note: UPDATED status will be not restored (S87 compat.)
*
static proc RestoreGetSysVars(saved)

   KillRead := saved[ GSV_KILLREAD ]
   BumpTop  := saved[ GSV_BUMPTOP ]
   BumpBot  := saved[ GSV_BUMPBOT ]
   LastExit := saved[ GSV_LASTEXIT ]
   LastPos  := saved[ GSV_LASTPOS ]
   GetActive( saved[ GSV_ACTIVEGET ] )
   ReadVar( saved[ GSV_READVAR ] )
   ReadProcName := saved[ GSV_READPROCNAME ]
   ReadProcLine := saved[ GSV_READPROCLINE ]
return

****************************************************
* GetReadVar()
* appends the NAME of the actual array-variable for READVAR()
*   with [elem] according to get:subscript
* Note: incompatible with Clipper'87 (here array name only)
*
static func GetReadVar(get as get)

local name := Upper(get:name)

#ifndef SUBSCRIPT_IN_READVAR                      // Clipper-87 compatib. ?
local i
#ifdef ADD_FS430
   local stat := fs_set ("zero", .F.)
   name := Upper(substr(get:name, 1, LEN(get:name)))
   fs_set ("zero", stat)
#endif

   if (!empty(get:subscript))
      for i := 1 to len(get:subscript)
         name += "[" + ltrim(str(get:subscript[i])) + "]"
      next
   end

#endif

return (name)


****************************************************
*  ShowScoreboard()  displays the actual INSERT status
*
static proc ShowScoreboard()

local nRow, nCol, cMsg

#ifdef FlagShip
   cMsg := _LANGNAMES (1)                         // get msg from FSsortab.xxx
#else
   cMsg := GSMSG_INSERT
#endif

   if ( Set(_SET_SCOREBOARD) )                        // SET SCOREBOARD ON ?
      nRow := Row()                                   // save actuall cursor
      nCol := Col()
      SetPos(SCORE_ROW, SCORE_COL)                    // see define
      DispOut( if(Set(_SET_INSERT), cMsg, SPACE(LEN(cMsg)) ))
      SetPos(nRow, nCol)                              // restore cursor
   end
return

****************************************************
*  DateMsg()   displays error on "invalid date"
*
static proc DateMsg()

local nRow, nCol, cMsg

#ifdef FlagShip
   cMsg := _LANGNAMES (3)                         // get msg from FSsortab.xxx
#else
   cMsg := GSMSG_INVDATE
#endif

   if ( Set(_SET_SCOREBOARD) )                        // SET SCOREBOARD ON ?
      nRow := Row()                                   // save actuall cursor
      nCol := Col()
      SetPos(SCORE_ROW, SCORE_COL)
      DispOut(cMsg)                                   // display "invalid.."
      SetPos(nRow, nCol)                              // reset cursor

      while ( Nextkey() == 0 )                        // wait for the next
      end                                             // key pressed

      SetPos(SCORE_ROW, SCORE_COL)                    // clear the message
      DispOut(SPACE(LEN(cMsg)))
      SetPos(nRow, nCol)
   end
return

/*** eof ***/

